/* parse.h */

#ifndef PARSE_DEF
#define PARSE_DEF

typedef struct keywords { /* top level keyword structure */
	char *keyword;
	int code;		/* flag code for parsing routine */
	int (*fcn)(FILE *fp, Machine *m, struct keywords *k);
} KeyWord;

typedef struct stmt_ent {	/* parse table for statements */
	char	*name;			/* statement name */
	int		code;			/* statement type */
	Stmt *(*fcn)(FILE *fp, Context *global, Context *local,
				struct stmt_ent *t);
} StmtEnt;

typedef struct op_ent {	/* parse table for operators */
	char	*name;			/* operator string */
	int		code;			/* operator binary code */
} OpEnt;

void parse_infile_statements(FILE *fp, Machine *machine);
int parse_decl_stmt(FILE *fp, Context *context, struct keywords *keyword);
void parse_local_decl_stmt(FILE *fp, Context *context, StmtEnt *stbl);

char *get_token(FILE *fp);
struct keywords *get_keyword(char *token);
void skip_to_character(FILE *fp, int ch);
void skip_to_end_of_statement(FILE *fp);
void get_line(char *buf, FILE *fp);
int get_nonblank_character(FILE *fp);
int get_alphanumeric_string(char *buf, FILE *fp);
int get_numeric_string(char *buf, FILE *fp);
int get_integer_value(char *buf, FILE *fp);
int get_float_value(char *buf, FILE *fp);
int get_string_value(char *buf, FILE *fp);

int parse_decl(FILE *fp, Machine *m, struct keywords *k);
int parse_fcn(FILE *fp, Machine *m, struct keywords *k);
Expr *parse_expression(FILE *fp, Context *global, Context *local);
void add_statement_to_context(Context *context, void *stmt);
void get_string_string(char *buf, FILE *fp);
Value *create_value_struct(void);
Value *create_value_for_token(char *name);
Value *create_value_for_number(char *name);
Value *create_value_for_operator(char *name);

void free_value(Value *val);
void discard_value(Value *val);

Block *parse_block(FILE *fp, Context *global, Context *local);
int encode_data_type(char *type);

#endif
